#include "midi_sequencer.h"

#include "audio/device/audio_device_manager.h"
#include "misc/tick.h"
#include "vst2/vst2_plugin_host.h"

IMPL_SINGLETON_INSTANCE(midi_sequencer)


void midi_sequencer::init(singleton_initliazer& initliazer) {
    singleton_t<midi_sequencer>::init(initliazer);
    auto adm = initliazer.require<audio_device_manager>();
    adm->on_sample_rate_changed.add_raw(this, &midi_sequencer::on_sample_rate_changed);
    set_bpm(140.0);
    set_tpq(960);
}

void midi_sequencer::process(double sample_rate, uint32_t delta_frames) {
    update_time(sample_rate, delta_frames);
    update_vst2_time_info();
}

void midi_sequencer::set_bpm(double in_bpm) {
    bpm_ = std::max(in_bpm, 1.0);
    vst2_time.tempo = bpm_;
}

void midi_sequencer::set_tpq(int32_t in_tpq) {
    tpq_ = in_tpq;
}

void midi_sequencer::on_sample_rate_changed(uint32_t sample_rate) {
    vst2_time.sampleRate = sample_rate;
}

void midi_sequencer::update_time(double sample_rate, uint32_t delta_frames) {
    audio_frame frame(delta_frames);
}

void midi_sequencer::update_vst2_time_info() {
    vst2_time.nanoSeconds = 0;
    vst2_time.ppqPos = tpq_pos_;
}

