#pragma once
#include <cstdint>

#include "misc/tick.h"
#include "misc/singleton/singleton.h"

class midi_sequencer : public singleton_t<midi_sequencer> {
public:

    void init(singleton_initliazer& initliazer) override;
    void process(double sample_rate, uint32_t delta_frames);

    const char* get_name() override { return "midi_sequencer"; }
    void set_bpm(double in_bpm);
    void set_tpq(int32_t in_tpq);
    [[nodiscard]] double get_bpm() const {
        return bpm_;
    }
    [[nodiscard]] int32_t get_tpq() const {
        return tpq_;
    }
protected:
    void on_sample_rate_changed(uint32_t sample_rate);
    void update_time(double sample_rate, uint32_t delta_frames);
private:
    void update_vst2_time_info();

    double bpm_ = 0;
    int32_t tpq_ = 0;
    audio_frame sample_pos_ = 0;
    int32_t tpq_pos_ = 0;
    double nano_seconds_ = 0;
};

DEFINE_SINGLETON_INSTANCE(midi_sequencer)
