#include "plugin_host.h"

#include "audio/mixer/channel_interface.h"
#include "window/window_manager.h"

plugin_host::~plugin_host() {
    delete channel;
}

void plugin_host::on_update_buffer_size(int buffer_size)
{
    ui_buffers = std::make_shared<circular_buffer_vector_type>();
    for (uint32_t i = 0; i < get_output_channels(); i++)
    {
        circular_buffer_type buffer(buffer_size * 3);
        ui_buffers->emplace_back(std::move(buffer));
    }
}

void plugin_host::try_open_editor() {
    if (!has_editor())
        return;
    if (editor_opened)
        return;
    create_and_open_editor();
    editor_opened = true;
}

void plugin_host::try_close_editor() {
    if (!has_editor())
        return;
    if (editor_opened == false)
        return;
    destroy_editor();
    editor_opened = false;
}

void plugin_host::toggle_editor() {
    if (!has_editor())
        return;
    if (editor_opened)
        try_close_editor();
    else
        try_open_editor();
}

void plugin_host::init_channel_interface() {
    channel = new channel_interface(get_input_channels(), get_output_channels());
}

void plugin_host::create_and_open_editor() {
    void* handle = get_window_manager()->create_plugin_window(this);
    if (!handle) {
        spdlog::error("Failed to create plugin window");
        return;
    }
    on_open_editor(handle);
}

void plugin_host::destroy_editor() {
    get_window_manager()->destroy_plugin_window(this);
}
