#pragma once
#include <cstdint>
#include <string>
#include <vector>
#include "extern.h"
#include "host_param.h"
#include "audio/misc/circular_audio_buffer.h"
#include "misc/delegates.h"

class mixer_track;
class channel_interface;

struct host_param_info {
    std::string name;
    std::string label;
    float min_value;
    float max_value;
};

class CORE_API plugin_host {
    friend class window_manager;
public:
    plugin_host() {
        static uint32_t id_counter = 0;
        id = ++id_counter;
    }
    virtual ~plugin_host();

    virtual bool load_plugin(const char* path) = 0;

    virtual void set_enabled(bool enabled) = 0;
    [[nodiscard]] virtual bool is_enabled() const = 0;

    virtual void on_update_sample_rate(double sample_rate) = 0;
    virtual void on_update_buffer_size(int buffer_size);
    [[nodiscard]] virtual uint32_t get_input_channels() const = 0;
    [[nodiscard]] virtual uint32_t get_output_channels() const = 0;
    virtual void update_channel_node_name() {}

    virtual void process(uint32_t frame_num) = 0;

    void try_open_editor();
    void try_close_editor();
    virtual void toggle_editor();
    virtual void idle_editor() {}

    [[nodiscard]] virtual bool has_editor() const { return false; }
    virtual void get_editor_size(uint32_t& width, uint32_t& height) const = 0;

    [[nodiscard]] virtual std::string load_name() const { return ""; }
    [[nodiscard]] virtual std::string load_vendor() const { return ""; }

    [[nodiscard]] virtual std::string get_parameter_name(int index) const = 0;
    [[nodiscard]] virtual std::string get_parameter_label(int index) const { return ""; };
    [[nodiscard]] virtual std::string get_parameter_display(int index) const { return ""; };
    [[nodiscard]] virtual int32_t get_param_count() const = 0;
    [[nodiscard]] virtual bool is_param_index_valid(const int32_t index) const { return index >= 0 && index < get_param_count(); }
    virtual void set_param_value(int32_t index, float value) = 0;
    virtual float get_param_value(int32_t index) = 0;
    virtual float get_param_min_value(int32_t index) { return 0.0f; }
    virtual float get_param_max_value(int32_t index) { return 1.0f; }

    void init_channel_interface();

    std::string name;
    std::string vendor;
    channel_interface* channel = nullptr;
    std::vector<mixer_track*> owner_tracks;

    [[nodiscard]] std::string get_imgui_id() const {
        return name + "##" + std::to_string(id);
    }
    std::shared_ptr<circular_buffer_vector_type> ui_buffers;
    multicast_delegate<void(int32_t index, float value)> on_param_changed;
protected:
    void create_and_open_editor();
    void destroy_editor();
    virtual void on_open_editor(void* window_handle) = 0;
    virtual void on_close_editor() = 0;
private:
    bool editor_opened = false;
    uint32_t id = 0;
};
