#pragma once
#include "audio/plugin_host/plugin_host.h"
#include "misc/lib_load.h"
#include "pluginterfaces/vst2.x/aeffect.h"
#include "pluginterfaces/vst2.x/aeffectx.h"

class vst2_plugin_host : public plugin_host {
public:
    static VstTimeInfo vst_time_info;
    vst2_plugin_host();
    ~vst2_plugin_host() override;
    bool load_plugin(const char* path) override;

    void set_enabled(bool enabled) override;
    [[nodiscard]] bool is_enabled() const override;

    void on_update_sample_rate(double sample_rate) override;
    void on_update_buffer_size(int buffer_size) override;
    [[nodiscard]] uint32_t get_input_channels() const override;
    [[nodiscard]] uint32_t get_output_channels() const override;
    void update_channel_node_name() override;

    void process(uint32_t frame_num) override;

    void idle_editor() override;
    [[nodiscard]] bool has_editor() const override;
    void get_editor_size(uint32_t& width, uint32_t& height) const override;
    // [[nodiscard]] ImVec2 get_editor_size() const override;
    [[nodiscard]] std::string get_parameter_name(int index) const override;
    [[nodiscard]] std::string get_parameter_label(int index) const override;
    [[nodiscard]] std::string get_parameter_display(int index) const override;
    [[nodiscard]] int32_t get_param_count() const override;
    void set_param_value(int32_t index, float value) override;
    float get_param_value(int32_t index) override;

    [[nodiscard]] std::string load_name() const override;
    [[nodiscard]] std::string load_vendor() const override;
protected:
    void on_open_editor(void* window_handle) override;
    void on_close_editor() override;
private:
    VstIntPtr dispatch(VstInt32 opcode, VstInt32 index = 0, VstIntPtr value = 0, void* ptr = nullptr, float opt = 0) const;
    AEffect* effect_;
    std::shared_ptr<dynamic_library> library_;
    bool enabled_ = false;
};

#define vst2_time vst2_plugin_host::vst_time_info
